/********************************************************************
   Based on the "Graphics Programming Black Book" by Michael Abrash 
   Changed to Actionscript and optimized to 3 vertices by 
   Ralf Bokelberg (actionscript@bokelberg.de) July 2001 
*********************************************************************/

/* usage: 
1. make three dragable mcs a1,a2 and a3  
2. make a line mc with a 45 degree line from 0,0 to 100,100
3. set the linkage of the line mc to export and name it "line"
4. include this script in frame 1 (#include "fill.as")
5. write the following script in frame 5

//fill the pointarrays with the actual values
PointX = [int(a1._x),int(a2._x),int(a3._x),int(a1._x)];
PointY = [int(a1._y),int(a2._y),int(a3._y),int(a1._y)];
//draw the fill
FillConvexPolygon(0,0,20);
//loop
gotoAndPlay(4);

*/

/* Color-fills a convex polygon. 
   "Convex" means that every horizontal line drawn through
   the polygon at any point would cross exactly two active edges
   (neither horizontal lines nor zero-length edges count as active
   edges; both are acceptable anywhere in the polygon), and that the
   right & left edges never cross. (It's OK for them to touch, though,
   so long as the right edge never crosses over to the left of the
   left edge.) Nonconvex polygons won't be drawn properly. Returns 1
   for success, 0 if memory allocation failed.
 */



/* Advances the index by one vertex forward through the vertex list,
   wrapping at the end of the list */
function INDEX_FORWARD(I){
   //return ++I & 3;	
   return ++I % PointX.length;
}

/* Advances the index by one vertex backward through the vertex list,
   wrapping at the start of the list */
function INDEX_BACKWARD(I){
   //return (I + 3) & 3;	
   return (--I + PointX.length) % PointX.length;
}

/* Advances the index by one vertex either forward or backward through
   the vertex list, wrapping at either end of the list */
function INDEX_MOVE(I, Dir){
   //return (I + Dir + 4)	& 3;

   if (Dir > 0){                
      return ++I % PointX.length;                      
   } else {                                                             
      return (--I + PointX.length) % PointX.length;
   }

}


function FillConvexPolygon(Depth){
   var i, minL, MaxIndex, minR, SkipFirst, Temp;
   var MinPoint_Y, MaxPoint_Y, TopIsFlat, LeftEdgeDir;
   var nex, cur, pre;
   var DeltaXN, DeltaYN, DeltaXP, DeltaYP;
   var XIdx,X1,Y1,X2,Y2;

   /* Scan the list to find the top and bottom of the polygon */
   MaxPoint_Y = MinPoint_Y = PointY[minL = MaxIndex = 0];
   for (i = 1; i < PointX.length; i++) {
      if (PointY[i] < MinPoint_Y){
         MinPoint_Y = PointY[minL = i]; /* new top */
      } else if (PointY[i] > MaxPoint_Y){
         MaxPoint_Y = PointY[MaxIndex = i]; /* new bottom */
      }   
   }
   if (MinPoint_Y == MaxPoint_Y) return(-1);  /* polygon is 0-height; avoid infinite loop below */

   /* Scan in ascending order to find the last top-edge point */
   minR = minL;
   while (PointY[minR] == MinPoint_Y)
      minR = INDEX_FORWARD(minR);
   minR = INDEX_BACKWARD(minR); /* back up to last top-edge point */

   /* Now scan in descending order to find the first top-edge point */
   while (PointY[minL] == MinPoint_Y)
      minL = INDEX_BACKWARD(minL);
   minL = INDEX_FORWARD(minL); /* back up to first top-edge point */

   /* Figure out which direction through the vertex list from the top
      vertex is the left edge and which is the right */
   LeftEdgeDir = -1; /* assume left edge runs down thru vertex list */
   if (TopIsFlat = (PointX[minL] != PointX[minR])){
      /* If the top is flat, just see which of the ends is leftmost */
      if (PointX[minL] > PointX[minR]) {
         LeftEdgeDir = 1;  /* left edge runs up through vertex list */
         Temp = minL;       /* swap the indices so minL   */
         minL = minR;  /* points to the start of the left */
         minR = Temp;       /* edge, similarly for minR   */
      }
   } else {
      /* Point to the downward end of the first line of each of the
         two edges down from the top */
      nex = INDEX_FORWARD(minR);
      pre = INDEX_BACKWARD(minL);
      /* Calculate X and Y lengths from the top vertex to the end of
         the first line down each edge; use those to compare slopes
         and see which line is leftmost */
      DeltaXN = PointX[nex] - PointX[minL];
      DeltaYN = PointY[nex] - PointY[minL];
      DeltaXP = PointX[pre] - PointX[minL];
      DeltaYP = PointY[pre] - PointY[minL];
      if ((DeltaXN * DeltaYP - DeltaYN * DeltaXP) < 0){
         LeftEdgeDir = 1;  /* left edge runs up through vertex list */
         Temp = minL;       /* swap the indices so minL   */
         minL = minR;  /* points to the start of the left */
         minR = Temp;       /* edge, similarly for minR   */
      }
   }

   /* Set the # of scan lines in the polygon, skipping the bottom edge
      and also skipping the top vertex if the top isn't flat because
      in that case the top vertex has a right edge component, and set
      the top scan line to draw, which is likewise the second line of
      the polygon unless the top is flat */
   if ((WorkingHLineList_length = MaxPoint_Y - MinPoint_Y - 1 + TopIsFlat) <= 0)
      return(-2);  /* there's nothing to draw, so we're done */
   YStart = YOffset + MinPoint_Y + 1 - TopIsFlat;

   /* Get memory in which to store the line list we generate */
   XStartL = new Array(WorkingHLineList_length);
   XEndL = new Array(WorkingHLineList_length);

   /* Scan the left edge and store the boundary points in the list */
   /* Initial pointer for storing scan converted left-edge coords */
   XIdx = 0;
   /* Start from the top of the left edge */
   cur = minL;
   /* Skip the first point of the first line unless the top is flat;
      if the top isn't flat, the top vertex is exactly on a right
      edge and isn't drawn */
   SkipFirst = ! TopIsFlat;
   /* Scan convert each line in the left edge from top to bottom */
   X1 = PointX[cur];
   Y1 = PointY[cur];
   do {
      //cur = (cur + LeftEdgeDir + 4) & 3;
      cur = INDEX_MOVE(cur, LeftEdgeDir);
      X2 = PointX[cur];
      Y2 = PointY[cur];
      XIdx = ScanEdge(X1,Y1,X2,Y2, SkipFirst, XIdx, XStartL);
      X1 = X2;
      Y1 = Y2;
      SkipFirst = 0; /* scan convert the first point from now on */
   } while (cur != MaxIndex);

   /* Scan the right edge and store the boundary points in the list */
   XIdx = 0;
   cur = minR;
   SkipFirst = ! TopIsFlat;
   /* Scan convert the right edge, top to bottom. X coordinates are
      adjusted 1 to the left, effectively causing scan conversion of
      the nearest points to the left of but not exactly on the edge */
   X1 = PointX[cur];
   Y1 = PointY[cur];
   LeftEdgeDir = -LeftEdgeDir;   
   do {
      //cur = (cur - LeftEdgeDir + 4) & 3;
      cur = INDEX_MOVE(cur, LeftEdgeDir);
      X2 = PointX[cur];
      Y2 = PointY[cur];
      XIdx = ScanEdge(X1,Y1,X2,Y2, SkipFirst, XIdx, XEndL);
      X1 = X2;
      Y1 = Y2;
      SkipFirst = 0; /* scan convert the first point from now on */
   } while (cur != MaxIndex);

   /* Draw the line list representing the scan converted polygon */
   Depth = DrawHorizontalLineList(YStart, XStartL, XEndL, Depth);

   /* Release the line list's memory and we're successfully done */
   delete(XStartL);
   delete(XEndL);
   return(Depth);
}

/* Scan converts an edge from (X1,Y1) to (X2,Y2), not including the
   point at (X2,Y2). This avoids overlapping the end of one line with
   the start of the next, and causes the bottom scan line of the
   polygon not to be drawn. If SkipFirst != 0, the point at (X1,Y1)
   isn't drawn. For each scan line, the pixel closest to the scanned
   line without being to the left of the scanned line is chosen */
function ScanEdge(X1, Y1, X2, Y2, SkipFirst, XIdx, XL){
   var X, Y, DeltaX, DeltaY;
   var InverseSlope, Yr;

   /* Calculate X and Y lengths of the line and the inverse slope */
   DeltaX = (X2 - X1);
   if ((DeltaY = Y2 - Y1) <= 0) return XIdx; /* guard against 0-length and horizontal edges */
   InverseSlope = DeltaX / DeltaY;
    	
   /* Store the X coordinate of the pixel closest to but not to the
      left of the line for each Y coordinate between Y1 and Y2, not
      including Y2 and also not including Y1 if SkipFirst != 0 */

   Yr = X1 + SkipFirst * InverseSlope;
   Y = SkipFirst
   while(Y++ < DeltaY){
      XL[XIdx++] = int(Yr);
      Yr += InverseSlope;
   }
   return XIdx;  /* advance caller's ptr */
}



function DrawHorizontalLineList(YStart, XStartL, XEndL, Depth){
	var i, l = XStartL.length, y, n;

	//delete old lines if we draw less
	while(--olddepth >= Depth) removeMovieClip(this["l" + olddepth]);
	
	//draw new lines 
	y=YStart + l;
	
	while(--l){
		n = "l" + Depth;
		if(Depth >= olddepth) //attach if we need new lines
			attachMovie("line", n, Depth++);
		with(this[n]){
			_x = XStartL[l];
			_y = --y;
			_xscale = XEndL[l] - _x;
		}	
	}
	olddepth = Depth;
}


